<?php
/* Eenos  License Provisioning Addon Module*
 * Licese : EPL (Eenos Public License)
 * Also, perform any initialization required by the service's library.
 */
// namespace WHMCS\Module\Addon\Eenoslicense;

use WHMCS\Database\Capsule;
use WHMCS\Module\Addon\Eenoslicense\Admin\AdminDispatcher;
// use WHMCS\Module\Addon\Eenoslicense\Client\ClientDispatcher;

if (!defined("WHMCS")) {
    die("This file cannot be accessed directly");
}
// Enable debug logs
define("EENOSLOG", false);
// A simple log module  for debug
function mylog($data){
    if (EENOSLOG) {
        $logfile=dirname(__FILE__).'/debug.log';
        if(is_array($data)) {
            file_put_contents($logfile,  print_r($data, true), FILE_APPEND);
        
        } else {
            file_put_contents($logfile, $data . "\n", FILE_APPEND);
        }
    }
     
}

/**
 * Define addon module configuration parameters.
 *
 * Includes a number of required system fields including name, description,
 * author, language and version.
 *
 * Also allows you to define any configuration parameters that should be
 * presented to the user when activating and configuring the module. These
 * values are then made available in all module function calls.
 *
 * Examples of each and their possible configuration parameters are provided in
 * the fields parameter below.
 *
 * @return array
 */
function eenoslicense_config()
{
    return [
        // Display name for your module
        'name' => 'Eenos License Management',
        // Description displayed within the admin interface
        'description' => 'This module provides provisioning of Eenos Licenses ',
        // Module author name
        'author' => "<a href='https://eenos.com/'>Eenos</a> Team",
        // Default language
        'language' => 'english',
        // Version number
        'version' => '1.0',
        'fields' => [
            // a text field type allows for single line text input
            'api_url' => [
                'FriendlyName' => 'API Base URL',
                'Type' => 'text',
                'Size' => '246',
                'Default' => 'https://manage.eenos.com/api',
                'Description' => 'Enter API Domain Link of license portal',
            ],
            'api_name' => [
                'FriendlyName' => 'API Name',
                'Type' => 'text',
                'Size' => '40',
                'Default' => '',
                'Description' => 'Enter API ID from license portal. Goto -> dashboard -> API Management -> List API',
            ],
            'api_id' => [
                'FriendlyName' => 'API Login ID',
                'Type' => 'text',
                'Size' => '40',
                'Default' => '',
                'Description' => 'Enter  User / API Login ID from license portal',
            ],
            // a password field type allows for masked text input
            'api_password' => [
                'FriendlyName' => 'API Password',
                'Type' => 'password',
                'Size' => '256',
                'Default' => '',
                'Description' => 'Enter secret api password value here',
            ],
            
        ]
    ];
}

/**
 * Activate.
 *
 * Called upon activation of the module for the first time.
 * Use this function to perform any database and schema modifications
 * required by your module.
 *
 * This function is optional.
 *
 * @see https://developers.whmcs.com/advanced/db-interaction/
 *
 * @return array Optional success/failure message
 */
function eenoslicense_activate()
{
    // Create custom tables and schema required by your module
    try {
        Capsule::schema()
            ->create(
                'mod_eenoslicense',
                function ($table) {
                    /** @var \Illuminate\Database\Schema\Blueprint $table */
                    $table->increments('id');
                    $table->string('filed');
                    $table->string('value');
                }
            );

        return [
            // Supported values here include: success, error or info
            'status' => 'success',
            'description' => 'Eenos license module has been activated',
        ];
    } catch (\Exception $e) {
        return [
            // Supported values here include: success, error or info
            'status' => "error",
            'description' => 'Unable to create eenos license  tables: ' . $e->getMessage(),
        ];
    }
}

/**
 * Deactivate.
 *
 * Called upon deactivation of the module.
 * Use this function to undo any database and schema modifications
 * performed by your module.
 *
 * This function is optional.
 *
 * @see https://developers.whmcs.com/advanced/db-interaction/
 *
 * @return array Optional success/failure message
 */
function eenoslicense_deactivate()
{
    // Undo any database and schema modifications made by your module here
    try {
        Capsule::schema()
            ->dropIfExists('mod_eenoslicense');

        return [
            // Supported values here include: success, error or info
            'status' => 'success',
            'description' => 'Eenos license module has been deactivated',
        ];
    } catch (\Exception $e) {
        return [
            // Supported values here include: success, error or info
            "status" => "error",
            "description" => "Unable to drop mod_eenoslicense: {$e->getMessage()}",
        ];
    }
}

/**
 * Upgrade.
 *
 * Called the first time the module is accessed following an update.
 * Use this function to perform any required database and schema modifications.
 *
 * This function is optional.
 *
 * @see https://laravel.com/docs/5.2/migrations
 *
 * @return void
 */
function eenoslicense_upgrade($vars)
{
    $currentlyInstalledVersion = $vars['version'];

    /// Perform SQL schema changes required by the upgrade to version 1.1 of your module
    if ($currentlyInstalledVersion < 1.1) {
        $schema = Capsule::schema();
        // Alter the table and add a new text column called "demo2"
        $schema->table('mod_eenoslicense', function($table) {
            $table->text('demo2');
        });
    }

    /// Perform SQL schema changes required by the upgrade to version 1.2 of your module
    if ($currentlyInstalledVersion < 1.2) {
        $schema = Capsule::schema();
        // Alter the table and add a new text column called "demo3"
        $schema->table('mod_eenoslicense', function($table) {
            $table->text('demo3');
        });
    }
}

/**
 * Admin Area Output.
 *
 * Called when the addon module is accessed via the admin area.
 * Should return HTML output for display to the admin user.
 *
 * This function is optional.
 *
 * @see AddonModule\Admin\Controller::index()
 *
 * @return string
 */
function eenoslicense_output($vars)
{
    mylog($vars);
    $api_url=$vars['api_url'];
    $api_id=$vars['api_id'];
    $api_password=$vars['api_password'];
    $version=$vars['version'];
    // Get common module parameters
    $modulelink = $vars['modulelink']; // eg. addonmodules.php?module=addonmodule
    $version = $vars['version']; // eg. 1.0
    $_lang = $vars['_lang']; // an array of the currently loaded language variables
    $loginurl=explode("api",$api_url)[0];
    // Get module configuration parameters
    $configTextField = $vars['API Login ID'];
    $configPasswordField = $vars['API Password'];
    // $configCheckboxField = $vars['Checkbox Field Name'];
    // $configDropdownField = $vars['Dropdown Field Name'];
    // $configRadioField = $vars['Radio Field Name'];
    // $configTextareaField = $vars['Textarea Field Name'];

    // Dispatch and handle request here. What follows is a demonstration of one
    // possible way of handling this using a very basic dispatcher implementation.

    // $action = isset($_REQUEST['action']) ? $_REQUEST['action'] : '';

    // $dispatcher = new AdminDispatcher();
    // $response = $dispatcher->dispatch($action, $vars);
    // echo $response;
    
    
    echo'<div class="container-flex">';
    echo '<small><p>Module Version : ' . $version. '</p></small>';
    echo <<< EOT
    <p>
    Eenos License management modules is used to manage Eenos  Licenses via API. To manage the licenses more easily, please login to the  Reseller License Portal.
    </p>
    <p>
    <button type="button" class="btn btn-primary"><a href="$loginurl" target="_blank" style="color: #ffffff">Login to Reseller Portal</a></button> 
    </p>
    EOT;
    echo '</div>';
}

