<?php
/**
 * WHMCS Eenos License  Provisioning Module
 *
 * For more information, please refer to the online documentation.
 *
 * @see https://docs.eenos.com/
 *
 * @copyright Copyright (c) eenos.com 2023
 * @license https://www.eenos.com/epl/ EPL
 */
if (!defined("WHMCS")) {
    die("This file cannot be accessed directly");
}
use WHMCS\Database\Capsule;
use WHMCS\Module\Server\CustomAction;
use WHMCS\Module\Server\CustomActionCollection;

// Enable debug logs
define("EENOSLOG", false);
// A simple log module  for debug
function eenoslicense_mylog($data){
    if (EENOSLOG) {
        $logfile=dirname(__FILE__).'/debug.log';
        if(is_array($data)) {
            file_put_contents($logfile,  print_r($data, true), FILE_APPEND);
        
        } else {
            file_put_contents($logfile, $data . "\n", FILE_APPEND);
        }
    }
     
}
//  To log use
// file_put_contents(dirname(__FILE__).'/log.txt', print_r($result, true)); 

function eenoslicense_MetaData()
{
    return array(
        'DisplayName' => 'Eenos License Provisioning',
        'APIVersion' => '1.1', // Use API Version 1.1
        'RequiresServer' => false, // Set true if module requires a server to work
    );
}

function eenoslicense_ConfigOptions()
{
    return [
        "SoftwareName" => [
            "FriendlyName" => "Eenos Software Name",
            "Type" => "text", # Text Box
            "Size" => "25", # Defines the Field Width
            "Description" => "Find Software Name from Dashboard -> Manage Packages -> List Packages -> Package Name",
            "Default" => "Eenos",
        ],
        "LicenseType" => [
            "FriendlyName" => "Eenos Server Type",
            "Type" => "radio", # Radio Selection of Options
            "Options" => "dedicated,vps",
            "Description" => "Chose the license for dedicated or vps server",
            "Default" => "dedicated",
        ],
    ];   
}

function eenoslicense_getAuth()
{
	$result=array();    
    $result['api_url']= Capsule::table("tbladdonmodules")->where("module", "eenoslicense")->where("setting", "api_url")->value("value");
    $result['api_id']= Capsule::table("tbladdonmodules")->where("module", "eenoslicense")->where("setting", "api_id")->value("value");
    $result['api_password'] = Capsule::table("tbladdonmodules")->where("module", "eenoslicense")->where("setting", "api_password")->value("value");
    // file_put_contents(dirname(__FILE__).'/log.txt', print_r($result, true)); 
	return($result);
}

function eenoslicense_getFieldId($packageid)
{    
    $field_id=Capsule::table("tblcustomfields")
            -> join('tblproducts','tblcustomfields.relid','=','tblproducts.id')
            -> where('tblproducts.id',$packageid)
            -> where('tblcustomfields.fieldname', 'like', 'LicenseID|%')
            ->value('tblcustomfields.id');    
    return $field_id;

}

function eenoslicense_PerformOperation(array $reseller,$action_url)
{    
	$statusCodes =array(
        "200" => "OK",
        "426" => "Your package limit exceeded",
        "208" => "License already exist",
        "400" => "Bad request, send invalid data to API",
        "204" => "Internal error, you need to contact support",
        "304" => "License Terminated,unable to suspend",
        "202" => "Package limit exceeded. you need to remove some licenses to downgrade the package",
        "404" => "License package not found",
        "204" => "Internal error, you need to contact support",
        "403" => "IP change only allowed once in 30 days",
        "207" => "You can only reissue a terminated license",
        "500" => "Failed",
        "700" => "php-curl is missing in this whmcs server",
    );
	$apiauth=eenoslicense_getAuth();
	$reseller['user']=$apiauth['api_id'];
    $reseller['password']=$apiauth['api_password'];
	$verifyurl=$apiauth['api_url'].$action_url;
	if (function_exists("curl_exec")) {
            $ch = curl_init();
            curl_setopt($ch, CURLOPT_URL, $verifyurl);
            curl_setopt($ch, CURLOPT_POST, 1);
            curl_setopt($ch, CURLOPT_POSTFIELDS, $reseller);
            curl_setopt($ch, CURLOPT_TIMEOUT, 300);
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
            $data = curl_exec($ch);
            if (curl_error($ch)) {
                throw new Exception('Unable to connect: ' . curl_errno($ch) . ' - ' . curl_error($ch));
            } elseif (empty($data)) {
                throw new Exception('Empty response');
            }
            curl_close($ch);
            $remoteresult=$data;
    }else{
            $remoteresult="{\"status\":\"500\",\"message\":\"php-curl is missing in this whmcs server\"}";
    }
	$output=json_decode($remoteresult,true);    
    if (is_null($output)) {
        throw new Exception('Invalid response format');
    }    

    if (isset($output['status'])){
        $result['success']=$output["status"];
    }else{
        if(isset($output["username"])){
            $output["status"]="200";
        }else{
            $output["status"]="500" ;           
        }
    }
    if (isset($output["detail"])){        
        $result['message']=$output['detail'];
    }else{        
        $result['message']=$output['message'];
    }
    $result['data']=$output;    
    return($result);

}

/**
 * Create an instance of a product/service.
 */

function eenoslicense_CreateAccount(array $params)
{
    try {        
        $reseller['operation']="Create";       
        $userinput=$params['customfields'];
        foreach($userinput as $field =>  $input) {
            $ip=trim($input);
        }
        $ip=$params['customfields']['LicenseIP'];
        $license_id=$params['customfields']['LicenseID'];
        $reseller['ip']=$ip;
        $reseller['software']=$params['configoption1'];
        $reseller['type']=$params['configoption2'];
        $action_url='/license/create/';        
        $result=eenoslicense_PerformOperation($reseller,$action_url);               
    } catch (Exception $e) {
        // Record the error in WHMCS's module log.
        logModuleCall(
            'eenoslicense',
            __FUNCTION__,
            $params,
            $e->getMessage(),
            $e->getTraceAsString()
        );

        return $e->getMessage();
    }   
    if($result['success']=="200"){
        $success = Capsule::table('tblhosting')
                    ->where('id', $params['serviceid'])
                    ->update(['domain' => $reseller['ip']]);        
        if (isset($result['data']['license_id'])){
            $license_id=trim($result['data']['license_id']);           
            $relid=$params['serviceid'];
            $filed_id=eenoslicense_getFieldId($params['packageid']);           
            $success_update=Capsule::table('tblcustomfieldsvalues')
                            ->updateOrInsert(
                                ['fieldid' => $filed_id, 'relid'=> $params['serviceid']],
                                ['value' => $license_id]
                            );
        }
        
        return 'success';
    }else{
        return($result['message']);
    }    
   
}

/**
 * Suspend an instance of a product/service.
 */

function eenoslicense_SuspendAccount(array $params)
{
    try {
        $reseller['operation']="Suspend";           
        $reseller['license_id']=$params['customfields']['LicenseID']; 
        $action_url='/license/suspend/';
        $result=eenoslicense_PerformOperation($reseller,$action_url);                
    } catch (Exception $e) {
        // Record the error in WHMCS's module log.
        logModuleCall(
            'eenoslicense',
            __FUNCTION__,
            $params,
            $e->getMessage(),
            $e->getTraceAsString()
        );

        return $e->getMessage();
    }
    if($result['success']=="200"){
        return 'success';
    }else{
        return($result['message']);
    }
    
}

/**
 * Un-suspend instance of a product/service.
 */
function eenoslicense_UnsuspendAccount(array $params)
{
    try {
        $reseller['operation']="Unsuspend"; 
        $reseller['license_id']=$params['customfields']['LicenseID']; 
        $action_url='/license/unsuspend/';        
        $result=eenoslicense_PerformOperation($reseller,$action_url);              
    } catch (Exception $e) {
        // Record the error in WHMCS's module log.
        logModuleCall(
            'eenoslicense',
            __FUNCTION__,
            $params,
            $e->getMessage(),
            $e->getTraceAsString()
        );

        return $e->getMessage();
    }    
    if($result['success']=="200"){
        return 'success';
    }else{
            return($result['message']);
    }   
}

/**
 * Terminate instance of a product/service.
 */
function eenoslicense_TerminateAccount(array $params)
{
    try {
        $reseller['operation']="Terminate";   
        $reseller['license_id']=$params['customfields']['LicenseID']; 
        $action_url='/license/terminate/';         
        $result=eenoslicense_PerformOperation($reseller,$action_url);          
    } catch (Exception $e) {
        // Record the error in WHMCS's module log.
        logModuleCall(
            'eenoslicense',
            __FUNCTION__,
            $params,
            $e->getMessage(),
            $e->getTraceAsString()
        );

        return $e->getMessage();
    }
    if($result['success']=="200"){
        return 'success';
    }else{
        return($result['message']);
    }   
   
}

/**
 * Renew an instance of a product/service.
 */
function eenoslicense_Renew(array $params)
{
    $status=$params['status'];
    if($status == "Active"){
        return 'success';
    }else{
        try {
        
            $reseller['operation']="Renew";  
            $reseller['license_id']=$params['customfields']['LicenseID']; 
            $action_url='/license/reissue/';       
            $result=eenoslicense_PerformOperation($reseller,$action_url);  
        } catch (Exception $e) {
            // Record the error in WHMCS's module log.
            logModuleCall(
                'eenoslicense',
                __FUNCTION__,
                $params,
                $e->getMessage(),
                $e->getTraceAsString()
            );
            return $e->getMessage();
        }
        if($result['success']=="200"){
            Capsule::table('tblhosting')
                ->where('id',$params['serviceid'] )            
                ->update(['domainstatus' => "Active"]);         
            return 'success';
        }else{
            return($result['message']);
        }
    }     
}

/**
 * Test connection with the given server parameters.
 */
function eenoslicense_TestConnection(array $params)
{
    try {        
        $reseller['operation']="TestConnection";                           
        $action_url='/login/';
        $result=eenoslicense_PerformOperation($reseller,$action_url); 
        $success = true;
        $errorMsg = '';
    } catch (Exception $e) {
        // Record the error in WHMCS's module log.
        logModuleCall(
            'eenoslicense',
            __FUNCTION__,
            $params,
            $e->getMessage(),
            $e->getTraceAsString()
        );

        $success = false;
        $errorMsg = $e->getMessage();
    }
    if($result['success']=="200"){
        return 'success';
    }else{
        return($result['message']);
    }
}

/**
 * Additional actions an admin user can invoke.
 *
 * Define additional actions that an admin user can perform for an
 * instance of a product/service.
 *
 * @see eenoslicense_buttonOneFunction()
 *
 * @return array
 */
function eenoslicense_AdminCustomButtonArray()
{
    return array(
        "Delete" => "DeleteLicense",        
        "Re-Issue" => "ReActivateLicense",
        "Change IP" => "ChangeLicenseIP",
    );
}

function eenoslicense_DeleteLicense(array $params)
{
    try {        
        $reseller['operation']="Delete"; 
        $reseller['license_id']=$params['customfields']['LicenseID']; 
        $action_url='/license/delete/'; 
        $result=eenoslicense_PerformOperation($reseller,$action_url);
    } catch (Exception $e) {       
        logModuleCall(
            'eenoslicense',
            __FUNCTION__,
            $params,
            $e->getMessage(),
            $e->getTraceAsString()
        );
        return $e->getMessage();
    }

    if($result['success']=="200"){
        Capsule::table('tblhosting')
            ->where('id',$params['serviceid'] )            
            ->update(['domainstatus' => "Terminated"]);        
        return 'success';
    }else{
        return($result['message']);
    }    
}

function eenoslicense_ReActivateLicense(array $params)
{
    try {        
        $reseller['operation']="Activate";  
        $reseller['license_id']=$params['customfields']['LicenseID']; 
        $action_url='/license/reissue/';        
        $result=eenoslicense_PerformOperation($reseller,$action_url);  
    } catch (Exception $e) {
        // Record the error in WHMCS's module log.
        logModuleCall(
            'eenoslicense',
            __FUNCTION__,
            $params,
            $e->getMessage(),
            $e->getTraceAsString()
        );
        return $e->getMessage();
    }
    if($result['success']=="200"){
        Capsule::table('tblhosting')
            ->where('id',$params['serviceid'] )            
            ->update(['domainstatus' => "Active"]);            
        return 'success';
    }else{
        return($result['message']);
    }    
}

function eenoslicense_ChangeLicenseIP(array $params)
{
    try {        
        $reseller['operation']="ChangeLicenseIP"; 
        $reseller['license_id']=$params['customfields']['LicenseID']; 
        $reseller['ip']=$params['customfields']['LicenseIP'];
        $reseller['force']='on';
        $action_url='/license/ipchange/';           
        $result=eenoslicense_PerformOperation($reseller,$action_url);            
    } catch (Exception $e) {
        // Record the error in WHMCS's module log.
        logModuleCall(
            'eenoslicense',
            __FUNCTION__,
            $params,
            $e->getMessage(),
            $e->getTraceAsString()
        );

        return $e->getMessage();
    }
    if($result['success']=="200"){
        $success = Capsule::table('tblhosting')
                    ->where('id', $params['serviceid'])
                    ->update(['domain' => $reseller['ip']]);        
        return 'success';
    }else{
        return($result['message']);
    }
    
}

function eenoslicense_ClientAreaCustomButtonArray()
{
    return array(
        "Client Change IP" => "clientChangeIP",
    );
}

function validateClientIP($ip)
{
    $pattern="/((^\s*((([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\.){3}([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5]))\s*$)|(^\s*((([0-9A-Fa-f]{1,4}:){7}([0-9A-Fa-f]{1,4}|:))|(([0-9A-Fa-f]{1,4}:){6}(:[0-9A-Fa-f]{1,4}|((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3})|:))|(([0-9A-Fa-f]{1,4}:){5}(((:[0-9A-Fa-f]{1,4}){1,2})|:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3})|:))|(([0-9A-Fa-f]{1,4}:){4}(((:[0-9A-Fa-f]{1,4}){1,3})|((:[0-9A-Fa-f]{1,4})?:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(([0-9A-Fa-f]{1,4}:){3}(((:[0-9A-Fa-f]{1,4}){1,4})|((:[0-9A-Fa-f]{1,4}){0,2}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(([0-9A-Fa-f]{1,4}:){2}(((:[0-9A-Fa-f]{1,4}){1,5})|((:[0-9A-Fa-f]{1,4}){0,3}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(([0-9A-Fa-f]{1,4}:){1}(((:[0-9A-Fa-f]{1,4}){1,6})|((:[0-9A-Fa-f]{1,4}){0,4}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(:(((:[0-9A-Fa-f]{1,4}){1,7})|((:[0-9A-Fa-f]{1,4}){0,5}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:)))(%.+)?\s*$))/";
    if (preg_match($pattern,$ip)){
        return TRUE;
    }else{
        return FALSE;
    }
}

function getOldIPid($ip,$relid)
{    
    $field_id = Capsule::table("tblcustomfieldsvalues")->where("relid",$relid)->where("value", $ip)->value("id");        
    return $field_id;
}

function eenoslicense_clientChangeIP(array $params)
{
    try {
        $newIP=trim(isset($_REQUEST['newip']) ? $_REQUEST['newip'] : '');
        $result=array();
        $old_ip=$params['customfields']['LicenseIP'];
        if (validateClientIP($newIP)){
            $reseller['license_id']=$params['customfields']['LicenseID']; 
            $reseller['ip']=$newIP;            
            $action_url='/license/ipchange/';           
            $result=eenoslicense_PerformOperation($reseller,$action_url);            
        }else{
            $result['success']="500";
            $result['message']="Invalid IP address";            
        }      
       
    } catch (Exception $e) {
        // Record the error in WHMCS's module log.
        logModuleCall(
            'eenoslicense',
            __FUNCTION__,
            $params,
            $e->getMessage(),
            $e->getTraceAsString()
        );

        return $e->getMessage();
    }
    if($result['success']=="200"){  
        Capsule::table('tblhosting')
            ->where('id', $params['serviceid'])
            ->update(['domain' => $reseller['ip']]); 
        $relid=$params['serviceid'];
        $id=getOldIPid($old_ip,$relid);
        if(!empty($id)){
            Capsule::table('tblcustomfieldsvalues')
                ->where('id',$id )                
                ->update(['value' => $newIP]);
        }

        return 'success';
    }else{
        return('-'.$result['message']);
    }    
}


/**
 * Client area output logic handling.
 * 
 */
function eenoslicense_ClientArea(array $params)
{
    // Determine the requested action and set service call parameters based on
    // the action.
    $requestedAction = isset($_REQUEST['customAction']) ? $_REQUEST['customAction'] : '';   
    // if ($requestedAction == 'manage') {
    //     $serviceAction = 'get_usage';
    //     $templateFile = 'templates/manage.tpl';
    // } else {
    //     $serviceAction = 'get_stats';
    //     $templateFile = 'templates/overview.tpl';
    // }
    $templateFile = 'templates/overview.tpl';
    try {
        // Call the service's function based on the request action, using the
        // values provided by WHMCS in `$params`.
        // if ($serviceAction=='change_ip'){
        //     $response=eenoslicense_clientChangeIP( $params,$newIP);
        //     file_put_contents('/home/dev/whmcs.eenos.com/modules/servers/eenoslicense/inputlog.txt', print_r($response, true));
        // }else{
        //     $response = array();
        // }
        $response = array();
        $extraVariable1 = 'eenos';
        $extraVariable2 = 'licmodule';

        return array(
            'tabOverviewReplacementTemplate' => $templateFile,
            'templateVariables' => array(
                'extraVariable1' => $extraVariable1,
                'extraVariable2' => $extraVariable2,
            ),
        );
    } catch (Exception $e) {
        // Record the error in WHMCS's module log.
        logModuleCall(
            'eenoslicense',
            __FUNCTION__,
            $params,
            $e->getMessage(),
            $e->getTraceAsString()
        );

        // In an error condition, display an error page.
        return array(
            'tabOverviewReplacementTemplate' => 'error.tpl',
            'templateVariables' => array(
                'usefulErrorHelper' => $e->getMessage(),
            ),
        );
    }
}


add_hook('ClientAreaPrimaryNavbar', 1, function ($menu)
{
    // Check whether the services menu exists.
    if (!is_null($menu->getChild('Services'))) {
        // Add a link to the module filter.
        $menu->getChild('Services')
            ->addChild(
                'My Eenos Licenses',
                array(
                    'uri' => 'clientarea.php?action=services&module=eenoslicense',
                    'order' => 15,
                )
            );
    }
});

add_hook('ClientAreaPrimarySidebar', 1, 'eenoslicense_sidebar_menu');
function eenoslicense_sidebar_menu($primarySidebar){
    $check_module = (!empty($GLOBALS['moduleparams']['moduletype']) ? strtolower($GLOBALS['moduleparams']['moduletype']) : '');
    if($check_module == 'eenoslicense'){
        if (!is_null($primarySidebar->getChild('Service Details Actions'))) {
        $primarySidebar->getChild('Service Details Actions')->removeChild('Custom Module Button Client Change IP');
        }
    }
}

